@@ -8,7 +8,7 @@ package Net::Async::IRC::Protocol;
 use strict;
 use warnings;
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 use base qw( IO::Async::Protocol::LineStream );
 
@@ -936,11 +936,10 @@ sub _on_message_text
    return $hints{handled};
 }
 
-# Keep perl happy; keep Britain tidy
-1;
-
-__END__
-
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
+
+=cut
+
+0x55AA;
